program exemple_readkey;

// |===========================================================================|
// | PROGRAM exemple_readkey                                                   |
// | F.BASSO 2020                                                              |
// |___________________________________________________________________________|
// | Logiciel de demonstration permettant de demander la sasie d'un code       |
// |___________________________________________________________________________|
// | Ce programme est libre, vous pouvez le redistribuer et ou le modifier     |
// | selon les termes de la Licence Publique Gnrale GNU publie par la       |
// | Free Software Foundation .                                                |
// | Ce programme est distribu car potentiellement utile,                     |
// | mais SANS AUCUNE GARANTIE, ni explicite ni implicite,                     |
// | y compris les garanties de commercialisation ou d'adaptation              |
// | dans un but spcifique.                                                   |
// | Reportez-vous  la Licence Publique Gnrale GNU pour plus de dtails.    |
// |                                                                           |
// | anbasso@wanadoo.fr                                                        |
// |___________________________________________________________________________|
// | Versions                                                                  |
// |  _________                                                                |
// | /  1.0.0  \______________________________________________________________ |
// || 2020/03/31 F.BASSO                                                      ||
// || Cration du programme                                                   ||
// ||_________________________________________________________________________||
// |===========================================================================|

{$APPTYPE CONSOLE}

{$R *.res}

uses
  System.SysUtils,
  Posix.Unistd,
  Posix.Base;

// #===========================================================================#
// #===========================================================================#
// #                                                                           #
// # Dclaration structure termios                                             #
// #                                                                           #
// #===========================================================================#
// #===========================================================================#

type
  cc_t = Byte;
  speed_t = Cardinal;
  tcflag_t = Cardinal;

const
  NCCS = 32;
  _POSIX_VDISABLE: cc_t = cc_t(#0);
  ICANON    = $0000002;
  ECHO      = $0000008;
  VMIN      = 6;
  TCSANOW   = 0;

type

  termios = record
    c_iflag  : tcflag_t;                        { modes d'entre }
    c_oflag  : tcflag_t;                        { modes de sortie }
    c_cflag  : tcflag_t;                        { modes de contrle }
    c_lflag  : tcflag_t;                        { modes locaux }
    c_line   : cc_t;                            { discipline de ligne }
    c_cc     : packed array[0..NCCS-1] of cc_t; { caractres de contrle }
    c_ispeed : speed_t;                         { vitesse d'entre }
    c_ospeed : speed_t;                         { vitesse de sortie }
  end;

// #===========================================================================#
// #===========================================================================#
// #                                                                           #
// # Dclaration des api                                                       #
// #                                                                           #
// #===========================================================================#
// #===========================================================================#

function getchar () : integer; cdecl; external libc name _PU + 'getchar';
//  ___________________________________________________________________________
// | function getchar                                                          |
// | _________________________________________________________________________ |
// || Permet de lire un octet depuis le stdin                                 ||
// ||_________________________________________________________________________||
// || Entres |                                                               ||
// ||         |                                                               ||
// ||_________|_______________________________________________________________||
// || Sorties | result : integer                                              ||
// ||         |   code caractre                                              ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

function tcsetattr( aIntFd : Integer; aIntOptionalActions : Integer; const aP_Termios : pointer): Integer; cdecl; external libc name _PU + 'tcsetattr';
//  ___________________________________________________________________________
// | function tcsetattr                                                        |
// | _________________________________________________________________________ |
// || Permet de definir les attributs TTY pour le descripteur de fichier      ||
// ||_________________________________________________________________________||
// || Entres | aIntFd : Integer                                              ||
// ||         |   descripteur de fichier                                      ||
// ||         | aIntOptionalActions : Integer                                 ||
// ||         |   options d'actions voir Constantes pour tcsetattr            ||
// ||         | const aP_Termios : pointer                                    ||
// ||         |   pointeur sur la structure termios contenant les attributs   ||
// ||_________|_______________________________________________________________||
// || Sorties | result : integer                                              ||
// ||         |   resultat de l'execution 0 ok -1 nok                         ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

function tcgetattr( aIntFd : Integer; aP_Termios: pointer): Integer; cdecl;external libc name _PU + 'tcgetattr';
//  ___________________________________________________________________________
// | Function tcgetattr                                                        |
// | _________________________________________________________________________ |
// || Permet de retourner les attributs TTY pour le descripteur de fichier    ||
// ||_________________________________________________________________________||
// || Entres | aIntFd : Integer                                              ||
// ||         |   descripteur de fichier                                      ||
// ||_________|_______________________________________________________________||
// || Sorties | aP_Termios: pointer                                           ||
// ||         |   pointeur sur la structure termios recevant les attributs    ||
// || Sorties | result : integer                                              ||
// ||         |   resultat de l'execution 0 ok -1 nok                         ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|


// #===========================================================================#
// #===========================================================================#
// #                                                                           #
// # Cration fonction                                                         #
// #                                                                           #
// #===========================================================================#
// #===========================================================================#

function readKey ( aBooMasqueCaractere : boolean = true ) : char;

//  ___________________________________________________________________________
// | function readKey                                                          |
// | _________________________________________________________________________ |
// || Permet de lire une touche venant du clavier                             ||
// ||_________________________________________________________________________||
// || Entres | aBooMasqueCaractere : boolean = true                          ||
// ||         |   mettre  true pour masquer le caractre                     ||
// ||_________|_______________________________________________________________||
// || Sorties | result : char                                                 ||
// ||         |   code du caractre appuy                                    ||
// ||_________|_______________________________________________________________||
// |___________________________________________________________________________|

var
  savedState : termios;
  newState   : termios;
  intMasque  : integer;
begin

// ****************************************************************************
// * Sauvegarde les attributs TTY                                             *
// ****************************************************************************

  tcgetattr( STDIN_FILENO, @savedState);
  newState := savedState;

// ****************************************************************************
// * Calcule le nouveau masque d'attribut, non canonical et mascage caractere *
// * si defini                                                                *
// ****************************************************************************

  intMasque := ICANON;
  if aBooMasqueCaractere then
    intMasque := intMasque or ECHO;

// ****************************************************************************
// * Defini le nouveau local mode                                             *
// ****************************************************************************

  newState.c_lflag := not( intMasque ) and newState.c_lflag;

// ****************************************************************************
// * Defini l'attente  un caractere                                          *
// ****************************************************************************

  newState.c_cc[VMIN] := 1;

// ****************************************************************************
// * Defini les nouveau attribut TTY                                          *
// ****************************************************************************

  tcsetattr(STDIN_FILENO, TCSANOW, @newState);

// ****************************************************************************
// * Attend l'appuie sur une touche                                           *
// ****************************************************************************

  result := char ( getchar());

// ****************************************************************************
// * restaurre les attributs TTY                                              *
// ****************************************************************************

  tcsetattr(STDIN_FILENO, TCSANOW, @savedState);

end;

var
  strCode : string;
  touche : char;
begin
  try
    write ('Sasissez votre code :' );
    touche := readKey( );
    while touche <> #10 do
    begin
      strCode := strCode + touche;
      touche := readKey( );
    end;
    writeln ('');
    writeln( 'votre code est : ' + strCode );
  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.
